#include "munixautostart.h"

#include <QCoreApplication>
#include <QDebug>
#include <QDir>
#include <QFileInfo>
#include <QRegularExpression>
#include <QTextStream>

class MUnixAutostart::Private
{
public:
    QString filename;
};

MUnixAutostart::MUnixAutostart(QObject *parent) :
    QObject(parent),
    d_ptr(new Private)
{
    if (QCoreApplication::applicationName().isEmpty()) {
        qWarning("You have to set the QCoreApplication::applicationName before instantiating UnixAutostart(). It won't do anything.");
        return;
    }

    d_ptr->filename = QString(QLatin1String("%1/.config/autostart/%2.desktop"))
                        .arg(qEnvironmentVariable("HOME"))
                        .arg(QCoreApplication::applicationName().remove(QRegularExpression(QLatin1String("\\s*"))));
}

MUnixAutostart::~MUnixAutostart()
{

}

bool MUnixAutostart::isEnabled() const
{
    if (d_ptr->filename.isEmpty()) {
        return false;
    }

    QFile file(d_ptr->filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    QString execLine = QString(QLatin1String("Exec=\"%1\"")).arg(QCoreApplication::applicationFilePath());
    QTextStream in(&file);
    while (!in.atEnd()) {
        QString line = in.readLine();
        if (line == execLine) {
            return true;
        }
    }

    return false;
}

void MUnixAutostart::setEnabled(bool enabled)
{
    if (d_ptr->filename.isEmpty()) {
        return;
    }

    if (isEnabled() == enabled) {
        return;
    }

    if (enabled) {
        QStringList lines;
        lines << QLatin1String("[Desktop Entry]");
        lines << QString(QLatin1String("Name=%1")).arg(QCoreApplication::applicationName());
        //lines << QString(QLatin1String("Icon=%1")).arg(QCoreApplication::applicationName());
        lines << QLatin1String("Hidden=false");
        lines << QLatin1String("Type=Application");
        lines << QLatin1String("X-GNOME-Autostart-enabled=true");
        lines << QString(QLatin1String("Exec=\"%1\"")).arg(QCoreApplication::applicationFilePath());
        lines << QString(QLatin1String(""));

        QDir(QFileInfo(d_ptr->filename).dir()).mkpath(QLatin1String("."));
        QFile autostartfile(d_ptr->filename);

        if (autostartfile.open(QFile::WriteOnly | QIODevice::Text)) {
            QTextStream out(&autostartfile);
            out << lines.join('\n');
        }

    } else {
        QFile::remove(d_ptr->filename);
    }

    if (enabled == isEnabled()) {
        emit isEnabledChanged(enabled);
    }
}

