#ifndef UNIXAUTOSTART_H
#define UNIXAUTOSTART_H

#include <QObject>
#include <QScopedPointer>

/*!
 * \brief The MUnixAutostart class
 *
 * This class can create and remove an autostart entry for the application
 * based on its QCoreApplication::applicationName()
 */
class MUnixAutostart : public QObject
{
    Q_OBJECT
    Q_PROPERTY(bool isEnabled READ isEnabled WRITE setEnabled NOTIFY isEnabledChanged)

public:
    /*!
     * \brief MUnixAutostart constructor
     *
     * Set the QCoreApplication::applicationName before instantiation
     */
    MUnixAutostart(QObject *parent = nullptr);
    ~MUnixAutostart();

signals:
    void isEnabledChanged(bool enabled);

public slots:
    /*!
     * \brief isEnabled
     * \return Returns whether an autostart entry for the application extsts or not
     *
     * \code
     * //You have to set the application name before you use this class
     * QCoreApplication::setApplicationName(QLatin1String("My amazing app"));
     *
     * // Will the application be started with you computer?
     * qDebug() << MUnixAutostart().isEnabled();
     * \endcode
     */
    bool isEnabled() const;

    /*!
     * \brief setEnabled
     * \param whether you want to enable or disable autostart
     *
     * \code
     * //You have to set the application name before you use this class
     * QCoreApplication::setApplicationName(QLatin1String("My amazing app"));
     *
     * // Create an autostart entry for the application
     * MUnixAutostart().setEnabled(true);
     * \endcode
     */
    void setEnabled(bool enabled);

private:
    class Private;
    QScopedPointer<Private> d_ptr;
};

#endif // UNIXAUTOSTART_H
