#include "munixmountsurveiler.h"

// Qt
#include <QDebug>
#include <QTextStream>
#include <QThread>

#include <fcntl.h> // open()
#include <sys/poll.h>
#include <unistd.h>

template<typename T>
inline void ignore_result(T /* unused result */) {}

class MUnixMountSurveiler::Private : public QObject
{
    Q_OBJECT

public:
    Private(QObject *parent = nullptr) :
        QObject(parent),
        running(true)
    {

    }

    ~Private()
    {
        running = false;
        ignore_result(write(writePipeFd, "x", 1));
        workerThread->wait();
    }

signals:
    void mountsChanged();

public:
    QScopedPointer<QThread> workerThread;
    bool running;
    int writePipeFd;
};

MUnixMountSurveiler::MUnixMountSurveiler(QObject *parent) :
    QObject(parent),
    d_ptr(new Private)
{
    connect(d_ptr.data(), &Private::mountsChanged, this, &MUnixMountSurveiler::mountsChanged, Qt::QueuedConnection);

    d_ptr->workerThread.reset(QThread::create([this]() {
        int mfd = open("/proc/mounts", O_RDONLY, 0);
        struct pollfd pfd[2];
        int rv;

        int pipefd[2];
        ignore_result(pipe(pipefd));
        d_ptr->writePipeFd = pipefd[1];

        pfd[1].fd = pipefd[0];
        pfd[1].events = POLLIN;
        pfd[1].revents = 0;

        QTextStream out(stdout);
        pfd[0].fd = mfd;
        pfd[0].events = POLLERR | POLLPRI;
        pfd[0].revents = 0;
        while ((rv = poll(&pfd[0], 2, -1)) >= 0) {
            if (!d_ptr->running) {
                break;
            }

            if (pfd[0].revents & POLLERR) {
              emit d_ptr->mountsChanged();
            }

            pfd[0].revents = 0;
        }
    }));

    d_ptr->workerThread->start();
}

MUnixMountSurveiler::~MUnixMountSurveiler()
{

}

#include "munixmountsurveiler.moc"
