#include "mhelpbrowser.h"
#include "ui_mhelpbrowser.h"

// Qt
#include <QDebug>
#include <QLineEdit>
#include <QSettings>

MHelpBrowser *MHelpBrowser::m_instance = nullptr;

MHelpBrowser::MHelpBrowser(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MHelpBrowser)
{
    if (m_instance != nullptr) {
        return;
    }
    m_instance = this;
    ui->setupUi(this);

    QString locale = QLocale::system().name();
    ui->textBrowser->setSearchPaths(QStringList() << QString(QLatin1String(":/help/%1")).arg(locale)
                                                  << QString(QLatin1String(":/help/%1")).arg(locale.split("_").first())
                                                  << QStringLiteral(":/help/en_US")
                                                  << QString(QLatin1String("help/%1")).arg(locale)
                                                  << QString(QLatin1String("help/%1")).arg(locale.split("_").first())
                                                  << QStringLiteral("help/en_US"));

    ui->action_Back->setEnabled(ui->textBrowser->isBackwardAvailable());
    ui->action_Forward->setEnabled(ui->textBrowser->isForwardAvailable());
    connect(ui->textBrowser, &QTextBrowser::backwardAvailable, ui->action_Back, &QAction::setEnabled);
    connect(ui->textBrowser, &QTextBrowser::forwardAvailable, ui->action_Forward, &QAction::setEnabled);

    connect(ui->textBrowser, &QTextBrowser::sourceChanged, [this](const QUrl &src) {
        Q_UNUSED(src)
        setWindowTitle(QString(QLatin1String("%1: %2 - %3")).arg(tr("Help")).arg(ui->textBrowser->documentTitle()).arg(QGuiApplication::applicationDisplayName()));
    });

    QWidget *wid = new QWidget(this);
    wid->setMinimumWidth(4);
    ui->toolBar->addWidget(wid);

    QLineEdit *location = new QLineEdit(this);
    location->setEnabled(false);
    location->setTextMargins(4,3,4,3);
    ui->toolBar->addWidget(location);
    connect(ui->textBrowser, &QTextBrowser::sourceChanged, [this, location](const QUrl &src) {
        location->setText(src.toString());
    });

    wid = new QWidget(this);
    wid->setMinimumWidth(4);
    ui->toolBar->addWidget(wid);

    ui->toolBar->addAction(ui->actionDecrease_font_size);
    ui->toolBar->addAction(ui->actionIncrease_font_size);

    QSettings settings;
    settings.beginGroup(QStringLiteral("HelpBrowser"));
    if (settings.contains(QStringLiteral("FontSize"))) {
        QFont browserFont = ui->textBrowser->font();
        browserFont.setPointSize(settings.value(QStringLiteral("FontSize")).toInt());
        ui->textBrowser->setFont(browserFont);
    }
}

MHelpBrowser::~MHelpBrowser()
{
    QSettings settings;
    settings.beginGroup(QStringLiteral("HelpBrowser"));
    settings.setValue(QStringLiteral("FontSize"), ui->textBrowser->font().pointSize());

    m_instance = nullptr;
    delete ui;
}

MHelpBrowser &MHelpBrowser::instance()
{
    if (m_instance == nullptr) {
        new MHelpBrowser;
    }
    return *m_instance;
}

void MHelpBrowser::open(const QString &filename)
{
    MHelpBrowser &inst = instance();

    if (filename.isEmpty()) {
        inst.ui->textBrowser->setSource(QStringLiteral("index.html"));
    } else {
        inst.ui->textBrowser->setSource(filename);
    }

    inst.show();
    inst.raise();
}

void MHelpBrowser::closeEvent(QCloseEvent *event)
{
    QMainWindow::closeEvent(event);
    deleteLater();
}

void MHelpBrowser::on_action_Back_triggered()
{
    ui->textBrowser->backward();
}

void MHelpBrowser::on_action_Forward_triggered()
{
    ui->textBrowser->forward();
}

void MHelpBrowser::on_actionDecrease_font_size_triggered()
{
    ui->textBrowser->zoomOut();
}

void MHelpBrowser::on_actionIncrease_font_size_triggered()
{
    ui->textBrowser->zoomIn();
}
