#include "qtkeychainkeystore.h"

#include "keychain.h"
#include "Core/stash.h"

using namespace QKeychain;

QtKeychainKeyStore::QtKeychainKeyStore(QObject *parent) : KeyStore(parent)
{

}

QtKeychainKeyStore::~QtKeychainKeyStore()
{

}

bool QtKeychainKeyStore::isAvailable()
{
    return true;
}

QString QtKeychainKeyStore::statusMessage()
{
    return QLatin1String();
}

void QtKeychainKeyStore::storePassword(QSharedPointer<Stash> stash, const QString &password)
{
    WritePasswordJob *job = new WritePasswordJob(QLatin1String("Molch Encfs Manger"));
    job->setKey(stash->stashDir());
    job->setTextData(password);

    job->connect(job, &Job::finished, [this, stash, password](QKeychain::Job *job) {
        if (job->error()) {
            emit storePasswordFinished(false, job->errorString());
        } else {
            emit storePasswordFinished(true);
        }
    });

    job->start();
}

void QtKeychainKeyStore::retrievePassword(QSharedPointer<Stash> stash)
{
    ReadPasswordJob *job = new ReadPasswordJob(QLatin1String("Molch Encfs Manger"));
    job->setKey(stash->stashDir());

    job->connect(job, &Job::finished, [this, stash](QKeychain::Job *job) {
        if (job->error()) {
            emit retrievePasswordFinished(false, job->errorString());
        } else {
            if (qobject_cast<ReadPasswordJob*>(job)->textData().isEmpty()) {
                emit retrievePasswordFinished(false, tr("The password couldn't be found in the keyring"));
            } else {
                emit retrievePasswordFinished(true, qobject_cast<ReadPasswordJob*>(job)->textData());
            }
        }
    });

   job->start();
}

void QtKeychainKeyStore::deletePassword(QSharedPointer<Stash> stash)
{
    DeletePasswordJob *job = new DeletePasswordJob(QLatin1String("Molch Encfs Manger"));
    job->setKey(stash->stashDir());

    job->connect(job, &Job::finished, [this, stash](QKeychain::Job *job) {
        if (job->error()) {
            emit deletePasswordFinished(false, job->errorString());
        } else {
            emit deletePasswordFinished(true);
        }
    });

    job->start();
}
