#ifndef ENCFS_H
#define ENCFS_H

#include <QObject>
#include <QSharedPointer>

class QProcess;
class Stash;

class Encfs : public QObject
{
    Q_OBJECT

public:
    explicit Encfs(QObject *parent = nullptr);
    ~Encfs();

    enum CreateStashFlag {
        DisableFilenameEncryption = 0x01
    };
    Q_DECLARE_FLAGS(CreateStashFlags, CreateStashFlag)
    Q_FLAG(CreateStashFlags)

    static bool isUserAllowOtherInFuseConf();

signals:
    void mountFinished(bool success, const QString &message = QLatin1String());
    void unmountFinished(bool success, const QString &message = QLatin1String());
    void createFinished(bool success, const QString &message = QLatin1String());
    void changePasswordFinished(bool success, const QString &message = QLatin1String());

public slots:
    void mount(const QSharedPointer<const Stash> stash, const QString &password);
    void unmount(const QSharedPointer<const Stash> stash);
    void create(const QSharedPointer<const Stash> stash, const QString &password, Encfs::CreateStashFlags flags = 0);
    void changePassword(const QSharedPointer<const Stash> stash, const QString &oldPassword, const QString &newPassword);
    QString info(const QSharedPointer<const Stash> stash);
    void cancel();

private:
    QStringList parseCombinedArgString(const QString &arguments);
    QSharedPointer<QProcess> m_encfsProcess;
    bool m_userCanceled;
};

#endif // ENCFS_H
