#include "configurestashpage.h"
#include "ui_configurestashpage.h"

// Qt
#include <QDebug>
#include <QFileDialog>

// Core
#include "Core/helpers.h"
#include "Core/settings.h"
#include "Core/stash.h"

#include <MHelpBrowser>

ConfigureStashPage::ConfigureStashPage(const QSharedPointer<Stash> &stash, QWidget *parent) :
    QWizardPage(parent),
    ui(new Ui::ConfigureStashPage),
    m_stash(stash),
    m_initialized(false),
    m_size(QSize(1,1))
{
    ui->setupUi(this);

    setPixmap(QWizard::LogoPixmap, QIcon::fromTheme(QLatin1String("document-edit")).pixmap(24,24));

    ui->label_help->setText(QString(QLatin1String("<html><a href=\"stashconfiguration.html\">%1</a></html>")).arg(tr("Help")));

    ui->lineEdit_mountDir->setMinimumHeight(ui->button_mountDir->height()-6);
}

ConfigureStashPage::~ConfigureStashPage()
{
    delete ui;
}

void ConfigureStashPage::setVisible(bool visible)
{
    if (visible && !m_initialized) {
        m_initialized = true;

        ui->button_sidebar->setChecked(Settings::instance().showSidebar());
        connect(ui->button_sidebar, &QPushButton::toggled, &Settings::instance(), &Settings::setShowSidebar);
        connect(&Settings::instance(), &Settings::showSidebarChanged, ui->button_sidebar, &QPushButton::setChecked);

        ui->lineEdit_mountDir->setText(m_stash->mountDir());
        connect(m_stash.data(), &Stash::mountDirChanged, this, &ConfigureStashPage::mountDirChanged);
        connect(ui->lineEdit_mountDir, &QLineEdit::textChanged, m_stash.data(), &Stash::setMountDir);

        checkMountDirChangable();
        connect(m_stash.data(), &Stash::isMountedChanged, this, &ConfigureStashPage::checkMountDirChangable);
        connect(m_stash.data(), &Stash::isBusyChanged, this, &ConfigureStashPage::checkMountDirChangable);

        ui->checkBox_mountOnStartup->setChecked(m_stash->mountOnStartup());
        connect(m_stash.data(), &Stash::mountOnStartupChanged, ui->checkBox_mountOnStartup, &QCheckBox::setChecked);
        connect(ui->checkBox_mountOnStartup, &QCheckBox::toggled, m_stash.data(), &Stash::setMountOnStartup);

        ui->checkBox_mountWhenAvailable->setChecked(m_stash->mountWhenAvailable());
        connect(m_stash.data(), &Stash::mountWhenAvailableChanged, ui->checkBox_mountWhenAvailable, &QCheckBox::setChecked);
        connect(ui->checkBox_mountWhenAvailable, &QCheckBox::toggled, m_stash.data(), &Stash::setMountWhenAvailable);

        ui->checkBox_openFileManager->setChecked(m_stash->openFileManager());
        connect(m_stash.data(), &Stash::openFileManagerChanged, ui->checkBox_openFileManager, &QCheckBox::setChecked);
        connect(ui->checkBox_openFileManager, &QCheckBox::toggled, m_stash.data(), &Stash::setOpenFileManager);

        ui->spinBox_unmountIdle->setEnabled(m_stash->unmountIdle());
        connect(m_stash.data(), &Stash::unmountIdleChanged, ui->spinBox_unmountIdle, &QSpinBox::setEnabled);

        ui->checkBox_unmountIdle->setChecked(m_stash->unmountIdle());
        connect(m_stash.data(), &Stash::unmountIdleChanged, ui->checkBox_unmountIdle, &QCheckBox::setChecked);
        connect(ui->checkBox_unmountIdle, &QCheckBox::toggled, m_stash.data(), &Stash::setUnmountIdle);

        ui->spinBox_unmountIdle->setValue(m_stash->unmountIdleTime());
        connect(m_stash.data(), &Stash::unmountIdleTimeChanged, ui->spinBox_unmountIdle, &QSpinBox::setValue);
        connect(ui->spinBox_unmountIdle, QOverload<int>::of(&QSpinBox::valueChanged), m_stash.data(), &Stash::setUnmountIdleTime);

        ui->checkBox_savePassword->setChecked(m_stash->savePassword());
        connect(m_stash.data(), &Stash::savePasswordChanged, ui->checkBox_savePassword, &QCheckBox::setChecked);
        connect(ui->checkBox_savePassword, &QCheckBox::toggled, m_stash.data(), &Stash::setSavePassword);

        ui->checkBox_persistentMountDirectory->setChecked(m_stash->persistentMountDirectory());
        connect(m_stash.data(), &Stash::persistentMountDirectoryChanged, ui->checkBox_persistentMountDirectory, &QCheckBox::setChecked);
        connect(ui->checkBox_persistentMountDirectory, &QCheckBox::toggled, m_stash.data(), &Stash::setPersistentMountDirectory);

        ui->checkBox_showInTray->setChecked(m_stash->showInTray());
        connect(m_stash.data(), &Stash::showInTrayChanged, ui->checkBox_showInTray, &QCheckBox::setChecked);
        connect(ui->checkBox_showInTray, &QCheckBox::toggled, m_stash.data(), &Stash::setShowInTray);

        ui->checkBox_trayIconIndicateMounted->setChecked(m_stash->trayIconIndicateMounted());
        connect(m_stash.data(), &Stash::trayIconIndicateMountedChanged, ui->checkBox_trayIconIndicateMounted, &QCheckBox::setChecked);
        connect(ui->checkBox_trayIconIndicateMounted, &QCheckBox::toggled, m_stash.data(), &Stash::setTrayIconIndicateMounted);
    }

    if (visible) {
        m_size = QWizardPage::minimumSizeHint();
    } else {
        m_size = QSize(1,1);
    }

    QWizardPage::setVisible(visible);
}

QSize ConfigureStashPage::sizeHint() const
{
    return m_size;
}

QSize ConfigureStashPage::minimumSizeHint() const
{
    return m_size;
}

void ConfigureStashPage::checkMountDirChangable()
{
    if (m_stash->isBusy() || m_stash->isMounted()) {
        ui->lineEdit_mountDir->setEnabled(false);
        ui->button_mountDir->setEnabled(false);
    } else {
        ui->lineEdit_mountDir->setEnabled(true);
        ui->button_mountDir->setEnabled(true);
    }
}

void ConfigureStashPage::on_button_mountDir_clicked()
{
    QFileDialog fileDialog(this);
    fileDialog.setDirectory(ui->lineEdit_mountDir->text().isEmpty() ? Helpers::pathToAbsolute(Settings::instance().baseDir()) : ui->lineEdit_mountDir->text());
    fileDialog.setFileMode(QFileDialog::Directory);
    fileDialog.setOptions(QFileDialog::ShowDirsOnly);

    if(fileDialog.exec()) {
       ui->lineEdit_mountDir->setText(fileDialog.selectedFiles().first());
    }
}

void ConfigureStashPage::mountDirChanged(const QString &mountDir)
{
    if (ui->lineEdit_mountDir->text() == mountDir) {
        return;
    }

    ui->lineEdit_mountDir->setText(mountDir);
}

void ConfigureStashPage::on_label_help_linkActivated(const QString &link)
{
    MHelpBrowser::open(link);
}
