#include "preferencesexpertpage.h"
#include "ui_preferencesexpertpage.h"

#include "Core/settings.h"

#include <MHelpBrowser>

PreferencesExpertPage::PreferencesExpertPage(QWidget *parent) :
    QWizardPage(parent),
    ui(new Ui::PreferencesExpertPage),
    m_initialized(false),
    m_size(QSize(1,1))
{
    ui->setupUi(this);

    setPixmap(QWizard::LogoPixmap, QIcon::fromTheme(QLatin1String("applications-system")).pixmap(24,24));

    ui->label_help->setText(QString(QLatin1String("<html><a href=\"advancedpreferences.html\">%1</a></html>")).arg(tr("Help")));
}

PreferencesExpertPage::~PreferencesExpertPage()
{
    delete ui;
}


void PreferencesExpertPage::setVisible(bool visible)
{
    if (visible && !m_initialized) {
        m_initialized = true;

        ui->button_sidebar->setChecked(Settings::instance().showSidebar());
        connect(ui->button_sidebar, &QPushButton::toggled, &Settings::instance(), &Settings::setShowSidebar);
        connect(&Settings::instance(), &Settings::showSidebarChanged, ui->button_sidebar, &QPushButton::setChecked);

        ui->lineEdit_fileManagerCommand->setText(Settings::instance().fileManagerCommand());
        connect(&Settings::instance(), &Settings::fileManagerCommandChanged, ui->lineEdit_fileManagerCommand, &QLineEdit::setText);
        connect(ui->lineEdit_fileManagerCommand, &QLineEdit::textChanged, &Settings::instance(), &Settings::setFileManagerCommand);

        ui->lineEdit_baseDir->setText(Settings::instance().baseDir());
        connect(&Settings::instance(), &Settings::baseDirChanged, ui->lineEdit_baseDir, &QLineEdit::setText);
        connect(ui->lineEdit_baseDir, &QLineEdit::textChanged, &Settings::instance(), &Settings::setBaseDir);

    }

    if (visible) {
        m_size = QWizardPage::minimumSizeHint();
    } else {
        m_size = QSize(1,1);
    }

    QWizardPage::setVisible(visible);
}

QSize PreferencesExpertPage::sizeHint() const
{
    return m_size;
}

QSize PreferencesExpertPage::minimumSizeHint() const
{
    return m_size;
}

void PreferencesExpertPage::on_label_help_linkActivated(const QString &link)
{
    MHelpBrowser::open(link);
}
