#include "removestashpage.h"
#include "ui_removestashpage.h"

// Qt
#include <QDebug>
#include <QIcon>

// Core
#include "Core/settings.h"
#include "Core/stash.h"
#include "Core/stashmanager.h"

#include <MHelpBrowser>

RemoveStashPage::RemoveStashPage(const QSharedPointer<Stash> &stash, QWidget *parent) :
    QWizardPage(parent),
    ui(new Ui::RemoveStashPage),
    m_stash(stash),
    m_initialized(false),
    m_size(QSize(1,1))
{
    ui->setupUi(this);

    setPixmap(QWizard::LogoPixmap, QIcon::fromTheme(QLatin1String("list-remove")).pixmap(24,24));

    connect(&StashManager::instance(), &StashManager::stashRemoved, this, &RemoveStashPage::stashRemoved);

    ui->label_help->setText(QString(QLatin1String("<html><a href=\"stashremove.html\">%1</a></html>")).arg(tr("Help")));
}

RemoveStashPage::~RemoveStashPage()
{
    delete ui;
}

void RemoveStashPage::setVisible(bool visible)
{
    if (visible && !m_initialized) {
        m_initialized = true;

        ui->button_sidebar->setChecked(Settings::instance().showSidebar());
        connect(ui->button_sidebar, &QPushButton::toggled, &Settings::instance(), &Settings::setShowSidebar);
        connect(&Settings::instance(), &Settings::showSidebarChanged, ui->button_sidebar, &QPushButton::setChecked);

        connect(m_stash.data(), &Stash::deleteFinished, this, &RemoveStashPage::deleteResult);

        connect(m_stash.data(), &Stash::isBusyChanged, this, &RemoveStashPage::validate);
        connect(m_stash.data(), &Stash::isAvailableChanged, this, &RemoveStashPage::validate);
        connect(m_stash.data(), &Stash::isMountedChanged, this, &RemoveStashPage::validate);
        connect(ui->radioButton_remove, &QRadioButton::toggled, this, &RemoveStashPage::validate);
        connect(ui->checkBox_delete, &QCheckBox::toggled, this, &RemoveStashPage::validate);
    }

    validate();

    if (visible) {
        m_size = QWizardPage::minimumSizeHint();
    } else {
        m_size = QSize(1,1);
    }

    QWizardPage::setVisible(visible);
}

void RemoveStashPage::on_button_remove_clicked()
{
    if (ui->radioButton_remove->isChecked()) {
        if (!m_stash->isBusy()) {
            StashManager::instance().removeStash(m_stash);
            qobject_cast<QDialog*>(parent()->parent())->close();
        }
    } else if (ui->radioButton_delete->isChecked() && ui->checkBox_delete->isChecked()) {
        StashManager::instance().delete_(m_stash);
    }
}

QSize RemoveStashPage::sizeHint() const
{
    return m_size;
}

QSize RemoveStashPage::minimumSizeHint() const
{
    return m_size;
}

void RemoveStashPage::validate()
{
    if (m_stash->action() == Stash::Action_Deleting) {
        ui->button_remove->setChecked(true);
        ui->button_remove->setEnabled(false);
        ui->radioButton_remove->setEnabled(false);
        ui->radioButton_delete->setEnabled(false);
        ui->checkBox_delete->setEnabled(false);
        return;
    }

    ui->radioButton_remove->setDisabled(m_stash->isBusy());

    ui->radioButton_delete->setEnabled((m_stash->isAvailable() && !m_stash->isBusy() && !m_stash->isMounted()));
    ui->checkBox_delete->setEnabled((m_stash->isAvailable() && !m_stash->isBusy() && !m_stash->isMounted() && ui->radioButton_delete->isChecked()));
    if (!(m_stash->isAvailable() && !m_stash->isBusy() && !m_stash->isMounted()) || ui->radioButton_remove->isChecked()) {
        ui->radioButton_remove->setChecked(true);
        ui->checkBox_delete->setChecked(false);
    }

    ui->button_remove->setEnabled((ui->radioButton_remove->isChecked() && ui->radioButton_remove->isEnabled()) ||
                                  (ui->radioButton_delete->isChecked() && ui->radioButton_delete->isEnabled() && ui->checkBox_delete->isChecked()));



    ui->button_remove->setChecked(m_stash->isBusy() && (m_stash->action() == Stash::Action_Deleting));
}

void RemoveStashPage::on_label_help_linkActivated(const QString &link)
{
    MHelpBrowser::open(link);
}

void RemoveStashPage::deleteResult(bool success, const QString &message)
{
    qDebug() << "delfinished"<< success;
    if (success) {
        qobject_cast<QDialog*>(parent()->parent())->close();
    } else {
        ui->label_message->setText(QString(QStringLiteral("<span style=\"color:#ff0000;\">✘</span> ")) + message);
    }
}

void RemoveStashPage::stashRemoved(const QSharedPointer<Stash> &stash)
{
    if (stash == m_stash) {
        qobject_cast<QDialog*>(parent()->parent())->close();
    }
}
